import {
  DATATABLE_PREFIX,
  JOIN_CHAR,
  DATATABLE_ID,
  DatatableJquery
} from './../../../src/components/datatable-jquery';

import { Repository } from 'aurelia-orm';
import { StageComponent, ComponentTester } from 'aurelia-testing';
import { bootstrap } from 'aurelia-bootstrapper';

import { Container, DOM } from 'aurelia-framework';
import { Config } from 'aurelia-api';

const ENDPOINT_NAME = 'github';
const ENDPOINT_PATH = 'https://api.github.com';

function getContainer(): Container {
  let container: Container = new Container();
  let config: Config = getApiConfig(container);

  configureEndpoints(config);

  return container;
}

function getRepository(container: Container): Repository {
  return new Repository(getApiConfig(container));
}

function getApiConfig(container: Container): Config {
  return container.get(Config);
}

function configureEndpoints(config: Config): void {
  config
    .registerEndpoint(ENDPOINT_NAME, ENDPOINT_PATH)
    .setDefaultEndpoint(ENDPOINT_NAME);
}

function fakeDatatables(count: number): NodeListOf<Element> {
  let fakeDOM = '';

  for (let index = 0; index < count; index++) {
    fakeDOM += '<div id="' + DATATABLE_PREFIX + JOIN_CHAR + index + '"></div>';
  }

  document.body.insertAdjacentHTML('afterbegin', fakeDOM);

  return document.querySelectorAll(
    '[id^=' + DATATABLE_PREFIX + JOIN_CHAR + ']'
  );
}

describe('Datatable jquery component', () => {
  let component: ComponentTester;
  let container: Container;
  let repository: Repository;

  beforeAll(() => {
    container = getContainer();

    repository = getRepository(container);
  });

  beforeEach(() => {
    component = StageComponent.withResources(
      '../src/components/datatable-jquery'
    )
      .inView(
        '<datatable-jquery repository.bind="repository"></datatable-jquery>'
      )
      .boundTo({
        id: 'myId',
        repository: repository
      });
  });

  it(`Aurelia lifecycle`, done => {
    component
      .manuallyHandleLifecycle()
      .create(bootstrap)
      .then(() => {
        spyOn(component.viewModel, 'configureDatatable');
        spyOn(component.viewModel, 'initializeDatatable');
        spyOn(component.viewModel, 'fetchData');

        component.attached();
      })
      .then(() => {
        expect(component.viewModel.configureDatatable).toHaveBeenCalled();
        expect(component.viewModel.initializeDatatable).toHaveBeenCalled();
        expect(component.viewModel.fetchData).toHaveBeenCalled();

        done();
      })
      .catch(console.error);
  });

  it(`Function | attached`, done => {
    component.create(bootstrap).then(() => {
      spyOn(component.viewModel, 'configureDatatable');
      spyOn(component.viewModel, 'initializeDatatable');
      spyOn(component.viewModel, 'fetchData');

      component.viewModel.attached();

      expect(component.viewModel.configureDatatable).toHaveBeenCalled();
      expect(component.viewModel.initializeDatatable).toHaveBeenCalled();
      expect(component.viewModel.fetchData).toHaveBeenCalled();

      done();
    });
  });

  it(`Get instancied datatables`, done => {
    let fakedDatatables = fakeDatatables(100);

    component.create(bootstrap).then(() => {
      expect(component.viewModel.getInstanciatedDatatables()).toEqual(
        fakedDatatables
      );

      done();
    });
  });

  it(`Get used ids`, done => {
    let DOMCount = 100;
    fakeDatatables(DOMCount);

    let desiredIds: number[] = [];

    for (let index = 0; index < DOMCount; index++) {
      desiredIds.push(index);
    }

    component.create(bootstrap).then(() => {
      expect(component.viewModel.getUsedIds()).toEqual(desiredIds);

      done();
    });
  });

  it(`Generate id for datatables`, done => {
    component.create(bootstrap).then(() => {
      let generatedId = component.viewModel.generateId();

      expect(generatedId).toBeDefined();
      expect(typeof generatedId).toBe('number');

      done();
    });
  });

  it(`Function | Should initialize datatable`, done => {
    component.create(bootstrap).then(() => {
      spyOn(component.viewModel, 'generateId');

      component.viewModel.initializeDatatable();

      expect(component.viewModel.generateId).toHaveBeenCalled();

      done();
    });
  });

  it(`Function | Should configure datatable`, done => {
    component.create(bootstrap).then(() => {
      spyOn(component.viewModel._datatableConfigurator, 'generateDatatableConfiguration');

      component.viewModel.configureDatatable();

      expect(
        component.viewModel._datatableConfigurator.generateDatatableConfiguration
      ).toHaveBeenCalled();

      done();
    });
  });

  it(`Constructor`, done => {
    let dataTableJquery = new DatatableJquery();

    expect(dataTableJquery.id).toEqual(DATATABLE_ID + -1);
    expect(dataTableJquery.data).toEqual([]);

    component.create(bootstrap).then(done);
  });

  afterEach(() => {
    component.dispose();

    document.body.innerHTML = '';
  });
});
